from graphviz import Digraph

def trace(root):
    #builds a set of all nodes and edges in a graph of Values starting from the root
    #nodes is a set of Values, edges are set of tuples that hold parent-child pairs
    nodes, edges = set(),set()
    def build(v):
        #v is a value
        if v not in nodes:
            nodes.add(v)
            for child in v._prev:
                edges.add((child,v))
                build(child)
    build(root)
    return nodes,edges

def draw_dot(root):
    dot = Digraph(format = 'svg', graph_attr = {'rankdir':'LR'}) #LR = left to right

    nodes,edges = trace(root)
    for n in nodes:
        uid = str(id(n))
        #for any value in the graph, create a rectangular 'record' node for it
        dot.node(name = uid,label = "{ data %s | grad %s }" % (n.data.tolist(), n.grad.tolist()),shape = 'record')
        if n._op:
            #if this value is the result of some operation create an op node for it
            dot.node(uid + n._op, n._op)
            #and connect it to the actual value node
            dot.edge(uid + n._op,uid)

    for n1,n2 in edges:
        #connect n1 to op node of n2
        dot.edge(str(id(n1)),str(id(n2)) + n2._op)
        

    return dot